<?php

namespace Drupal\auto_translation;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Logger\LoggerChannelTrait;
use Drupal\Core\Render\Markup;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\rest\ResourceResponse;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\RequestException;
use Google\Cloud\Translate\V2\TranslateClient;

/**
 * Utility class for auto_translation module functions.
 *
 * @package Drupal\auto_translation
 */
class Utility {

  use StringTranslationTrait;
  use LoggerChannelTrait;

  /**
   * Custom function to make API call to JW Player V2 API.
   *
   * @var string
   * The X key value.
   *
   * @var string
   * The Y key value.
   *
   * @var string
   * The endpoint value.
   *
   * @var string
   * The method value.
   *
   * @return string|null
   *   The array of jw media.
   */
  public function translateApiBrowserCall($text, $s_lang, $t_lang) {
    $client = new Client();
    $logger = $this->getLogger('auto_translation');
    $httpclient_options = [];
    $httpclient_options['verify'] = FALSE;
    $httpclient_options['cache-control'] = 'no-cache';
    $translation = null;
    $googleEndpoint = 'https://translate.googleapis.com/translate_a/single?client=gtx&sl=';
    try {
      $response = $client->get($googleEndpoint . $s_lang . '&tl=' . $t_lang . '&dt=t&q=' . $text, $httpclient_options);
      $data = $response->getBody()->getContents();
      $result = Json::decode($data);
      $string = '';
      if (isset($result[0]) && !empty($result[0])) {
        foreach ($result[0] as $key => $value) {
          if ($value !== NULL && $value[0] && !empty($value[0])) {
            $string .= $value[0];
          }
        }
      }
      $translation = $string && !empty($string) && $string !== '' ? $string : null;
    } catch (RequestException $exception) {
      $output =  urldecode($exception->getMessage());
      $to_remove = $this->getStringBetween($output, 'error: `', '` ');
      $path_remove = explode("response:", str_replace($to_remove, '', $output));
      $outputUpdated = $path_remove[0];
      $message = str_replace('`', '', $path_remove[0]);
      $error_code = $this->getStringBetween($output, 'a `', ' ');
      $reason = $this->getStringBetween($outputUpdated, $error_code, '`');

      $error = [
        'Error code' => $error_code ? $error_code : '400',
        'Error message' => $message ? $message : 'default error message',
        'Error reason' => $reason ? strtolower($reason) : 'default error reason',
      ];
      $logger->error('Auto translation error: @error', ['@error' => json_encode($error)]);
    }
      return $translation;
  }

  /**
   * Custom function to return saved resources.
   */
  public function getEnabledContentTypes() {
    $config = $this->config();
    $enabledContentTypes = $config->get('auto_translation_content_types') ? $config->get('auto_translation_content_types') : NULL;
    return $enabledContentTypes;
  }

  /**
   * Custom get string beetwin function.
   */
  public function getStringBetween($string, $start, $end) {
    $string = ' ' . $string;
    $ini = strpos($string, $start);
    if ($ini == 0)
      return '';
    $ini += strlen($start);
    $len = strpos($string, $end, $ini) - $ini;
    return substr($string, $ini, $len);
  }

  /**
   * Retrieves the container.
   *
   * @return mixed
   *   The container.
   */
  public static function getContainer() {
    return \Drupal::getContainer();
  }

  /**
   * Retrieves the configuration settings.
   *
   * @return object
   *   The configuration settings.
   */
  public static function config() {
    return static::getContainer()
      ->get('config.factory')
      ->get('auto_translation.settings');
  }

  /**
   * Returns the path of the module.
   *
   * @return string
   *   The path of the module.
   */
  public static function getModulePath() {
    return static::getContainer()
      ->get('extension.list.module')
      ->getPath('auto_translation');
  }

  /**
   * Retrieves the specified module by name.
   *
   * @param string $module_name
   *   The name of the module to retrieve.
   *
   * @return mixed|null
   *   The module object if found, null otherwise.
   */
  public static function getModule($module_name) {
    return static::getContainer()
      ->get('extension.list.module')
      ->get($module_name);
  }
}
